@@ -10,7 +10,7 @@ use strict;
 use Lemonldap::NG::Handler::Simple qw(:apache :headers :traces);
 use LWP::UserAgent;
 
-our $VERSION = '0.992';
+our $VERSION = '1.2.0';
 
 ##########################################
 # COMPATIBILITY WITH APACHE AND APACHE 2 #
@@ -89,8 +89,10 @@ sub run($$) {
     if ( $r->method eq "POST" ) {
         my $len = $r->headers_in->{'Content-Length'};
         my $buf;
-        $r->read( $buf, $len );
-        $request->content($buf);
+        if ($len) {
+            $r->read( $buf, $len );
+            $request->content($buf);
+        }
     }
     $headers_set = 0;
 
@@ -130,13 +132,24 @@ sub headers {
     # Scan LWP response headers to generate Apache response headers
     my ( $location_old, $location_new ) = split /[;,]+/,
       $r->dir_config('LmLocationToReplace');
+    my ( $cookieDomain_old, $cookieDomain_new ) = split /[;,]+/,
+      $r->dir_config('LmCookieDomainToReplace');
+
     $response->scan(
         sub {
 
             # Replace Location headers
             $_[1] =~ s#$location_old#$location_new#o
               if ( $location_old and $location_new and $_[0] =~ /Location/i );
+
+            # Replace Set-Cookie headers
+            $_[1] =~ s#$cookieDomain_old#$cookieDomain_new#o
+              if (  $cookieDomain_old
+                and $cookieDomain_new
+                and $_[0] =~ /Set-Cookie/i );
+
             lmSetErrHeaderOut( $r, @_ );
+
             $class->lmLog(
                 "$class: header pushed to the client: " . $_[0] . ": " . $_[1],
                 'debug'
@@ -166,7 +179,8 @@ apache/conf/httpd.conf:
   SetHandler perl-script
   PerlHandler Lemonldap::NG::Handler::Proxy
   PerlSetVar LmProxyPass http://real-server.com/
-  PerlSetVar LmLocationToReplace http://real-server/,https://lemon.server/
+  PerlSetVar LmLocationToReplace http://real-server.com/,https://lemon.server/
+  PerlSetVar LmCookieDomainToReplace real-server.com,lemon.server
 
   # Or just on a Location
   PerlModule Lemonldap::NG::Handler::Proxy
@@ -174,7 +188,8 @@ apache/conf/httpd.conf:
     SetHandler perl-script
     PerlHandler Lemonldap::NG::Handler::Proxy
     PerlSetVar LmProxyPass https://real-server.com/
-    PerlSetVar LmLocationToReplace http://real-server/,https://lemon.server/
+    PerlSetVar LmLocationToReplace http://real-server.com/,https://lemon.server/
+    PerlSetVar LmCookieDomainToReplace real-server.com,lemon.server
   </Location>
 
 =head1 DESCRIPTION
@@ -191,6 +206,9 @@ manage redirections if the remote host use it without the good domain.
 =item * B<LmLocationToReplace> (optional): substitution to do to avoid bad
 redirections. See synopsys for usage.
 
+=item * B<LmCookieDomainToReplace> (optional): substitution to do to set cookies
+from proxied application. See synopsys for usage.
+
 =back
 
 =head2 EXPORT
@@ -200,7 +218,7 @@ None by default.
 =head1 SEE ALSO
 
 Lemonldap::NG::Handler(3), LWP::UserAgent,
-http://wiki.lemonldap.objectweb.org/xwiki/bin/view/NG/Presentation
+L<http://lemonldap-ng.org/>
 
 =head1 AUTHOR
 
@@ -209,7 +227,7 @@ Xavier Guimard, E<lt>x.guimard@free.frE<gt>
 =head1 BUG REPORT
 
 Use OW2 system to report bug or ask for features:
-L<http://forge.objectweb.org/tracker/?group_id=274>
+L<http://jira.ow2.org>
 
 =head1 DOWNLOAD
 
@@ -221,7 +239,7 @@ L<http://forge.objectweb.org/project/showfiles.php?group_id=274>
 Copyright (C) 2005, 2007, 2010 by Xavier Guimard E<lt>x.guimard@free.frE<gt>
 
 This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself, either Perl version 5.8.4 or,
+it under the same terms as Perl itself, either Perl version 5.10.0 or,
 at your option, any later version of Perl 5 you may have available.
 
 =cut
